Attribute VB_Name = "modDatabase"
' Mike Gagne
' MikeGagne@OrthoRX.NET

' modDatabase: contains all Database functions needed for this project
' Note that Oracle cannot have autonumber reset, but since I use the
' OpenDatabase, CloseDatabase in every DB project I make, I left it in.

' I created all functions in this module, with the exception of a few
' lines of code I used in the AutoNumber reset function which I
' grabbed off the Access help file, I believe.

Public Enum DBType    ' type of database
  dbAccess2000 = 0
  dbAccess97 = 1
  dbOracle = 2
End Enum

Public Function OpenDatabase(DatabaseType As DBType, dbSource As String, Optional UserID As String, Optional Password As String) As ADODB.Connection
  ' Opens an ADO Connection to a specified Database
  ' Calling syntax: Set MyConnObj = OpenDatabase(DBType, Source[, User][,Pass])

  Dim sConnString   As String   ' database connection string
  Dim sUserPass     As String   ' user name and password
  Dim sProvider     As String   ' data provider for OLEDB

  ' if user id was passed, then construct the user/pass string
  If UserID <> "" Then
    sUserPass = "User ID=" & UserID & ";Password=" & Password & ";"
  End If

  ' decide what kind of database we're working with
  Select Case DatabaseType
    Case Is = dbAccess2000  ' use Jet 4.0 for Access 2000 and 2002
      sProvider = "PROVIDER=Microsoft.Jet.OLEDB.4.0;"
    Case Is = dbAccess97    ' use Jet 3.51 for Access 97
      sProvider = "PROVIDER=Microsoft.Jet.OLEDB.3.51;"
    Case Is = dbOracle      ' use MSDAORA.1 for Oracle (Microsoft's provider)
      sProvider = "PROVIDER=MSDAORA.1;"
  End Select

  ' construct the connection string
  sConnString = sProvider & "Data Source=" & dbSource & ";" & sUserPass
  ' set the connection object and open the connection to the database
  Set OpenDatabase = New ADODB.Connection
  OpenDatabase.Open sConnString
End Function

Public Function CloseDatabase(ByRef dbClose As ADODB.Connection) As Boolean
  ' Closes a connection object
  ' Calling Syntax: CloseDatabase MyConnObj
  CloseDatabase = False
  ' if the connection is open, then close it
  If dbClose.State = adStateOpen Then
    dbClose.Close
    CloseDatabase = True
  End If
  Set dbClose = Nothing   ' free the memory
End Function

Public Function ResetSeed(cnn As ADODB.Connection, sTable As String)
  ' Resets the seed of a table in a specified database
  Dim col           As ADOX.Column
  Dim cat           As New ADOX.Catalog
  Dim rstColumns    As New ADODB.Recordset
  Dim rstMaxValue   As New ADODB.Recordset
  Dim strCol        As String
  Dim lngSeed       As Long

  ' set the active connection of the catalog object
  cat.ActiveConnection = cnn

  ' update the main form's statusbar text (take this out if you plan on using
  ' this module elsewhere)
  frmMain.Status.SimpleText = "Setting Autonumber Seed for " & sTable & "..."
  DoEvents

  ' open the list of columns in the specified table
  Set rstColumns = cnn.OpenSchema(adSchemaColumns, Array(Empty, Empty, sTable))
  Set rstMaxValue.ActiveConnection = cnn  ' initialize the max value recordset

  ' loop through each column in the table
  Do Until rstColumns.EOF
    ' set the name of the column
    strCol = rstColumns.Fields("COLUMN_NAME")
    ' set the column object
    Set col = cat.Tables(sTable).Columns(strCol)
    ' if the column is AutoNumber field, then...
    If CBool(col.Properties("AutoIncrement")) = True Then
      ' get the max value of the column
      rstMaxValue.Open "SELECT MAX(" & strCol & ") FROM [" & sTable & "]"
      ' if the max doesn't exist then seed = 0, otherwise, it's the max + 1
      lngSeed = IIf(IsNull(rstMaxValue.Fields(0).Value), 0, rstMaxValue.Fields(0).Value + 1)
      rstMaxValue.Close   ' close the recordset
      If col.Properties("Seed") <> lngSeed Then
        ' if the Seed is not already correct, then update it and refresh
        col.Properties("Seed") = lngSeed
        cat.Tables(sTable).Columns.Refresh
      End If
    End If
    rstColumns.MoveNext   ' next record (next column)
  Loop
  rstColumns.Close  ' close the recordset

  ' free up the memory
  Set rstColumns = Nothing
  Set rstMaxValue = Nothing
  Set col = Nothing
  Set cat = Nothing
End Function

Public Function CompactRepair(sDestination As String)
  ' Compacts and Repairs a Microsoft Access Database
  ' Function receives a string equal to the full path and filename of the DB
  Dim JRO       As New JRO.JetEngine    ' jet engine object
  Dim sTempFile As String               ' temp file
  Dim sExt      As String               ' extension of the file

  ' get the file extension
  sExt = Right$(sDestination, Len(sDestination) - InStrRev(sDestination, ".") - 1)
  ' set the temp file name
  sTempFile = Left$(sDestination, Len(sDestination) - Len(sExt)) & "-Compacting" & sExt
  ' rename the destination to the temp file name
  Name sDestination As sTempFile

  ' compact the old database into the destination database file name
  JRO.CompactDatabase "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & sTempFile, _
    "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & sDestination & ";Jet OLEDB:Engine Type=5"
  ' remove the temp file
  Kill sTempFile
End Function
